@@ -1,12 +1,9 @@
 /*
- * -----------------------------------------------------------------
- * $Revision$
- * $Date$
  * -----------------------------------------------------------------
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2020, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -367,6 +364,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
   CVodeMem cv_mem;
   CkpntMem tmp;
   DtpntMem *dt_mem;
+  long int nstloc;
   int flag, i;
   booleantype allocOK, earlyret;
   realtype ttest;
@@ -497,13 +495,25 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
   }
 
   /* Integrate to tout (in CV_ONE_STEP mode) while loading check points */
+  nstloc = 0;
   for(;;) {
 
+    /* Check for too many steps */
+
+    if ( (cv_mem->cv_mxstep>0) && (nstloc >= cv_mem->cv_mxstep) ) {
+      cvProcessError(cv_mem, CV_TOO_MUCH_WORK, "CVODEA", "CVodeF",
+                     MSGCV_MAX_STEPS, cv_mem->cv_tn);
+      flag = CV_TOO_MUCH_WORK;
+      break;
+    }
+
     /* Perform one step of the integration */
 
     flag = CVode(cv_mem, tout, yout, tret, CV_ONE_STEP);
     if (flag < 0) break;
 
+    nstloc++;
+
     /* Test if a new check point is needed */
 
     if ( cv_mem->cv_nst % ca_mem->ca_nsteps == 0 ) {
@@ -2026,8 +2036,9 @@ static int CVAdataStore(CVodeMem cv_mem, CkpntMem ck_mem)
   /* Run CVode to set following structures in dt_mem[i] */
   i = 1;
   do {
-    // Modified for AMICI
-    // (dt_mem has dimension ca_mem->ca_nsteps)
+    /* Modified for AMICI
+     * (dt_mem has dimension ca_mem->ca_nsteps)
+     */
     if (i > ca_mem->ca_nsteps) return(CV_FWD_FAIL);
 
     flag = CVode(cv_mem, ck_mem->ck_t1, ca_mem->ca_ytmp, &t, CV_ONE_STEP);
